//
//  SquishProgressController.m
//  SquishTools
//
//  Created by Da Woon Jung on 05/05/16.
//  Copyright 2005. All rights reserved.
//

#import "SquishProgressController.h"


@implementation SquishProgressController

- (IBAction)doProfileEmbedded:(id)sender
{
    if (sender)
        _profileChoice = [sender tag];
}

- (IBAction)doProfileOverride:(id)sender
{
    if (sender)
        _profileChoice = [sender tag];

    _profileOverrideChoice = [profileOverrideMenu selectedTag];
}

- (IBAction)doReplace:(id)sender
{
    if (sender)
        _replaceChoice = [sender tag];
}

- (IBAction)doStop:(id)sender
{
    _replaceChoice = kSquishStop;
    _profileChoice = kSquishStop;
    
    [NSApp abortModal];
}


- (id)init
{
    self = [super initWithWindowNibName: @"Progress"];
    return self;
}

- (void)startProgress
{
    [replaceConfirmApplyAll setState: NSOffState];
    _replaceChoice = kSquishNoChoice;

    if (!_modalSession)
    {
        _modalSession = [NSApp beginModalSessionForWindow: [self window]];
    }
}

- (void)endProgress
{
    if (_modalSession)
    {
        [NSApp endModalSession: _modalSession];
        _modalSession = nil;
    }
}


- (BOOL)runModalLoop
{
    BOOL result = YES;
    if (_modalSession)
    {
        if ([NSApp runModalSession: _modalSession] != NSRunContinuesResponse)
            result = NO;
    }
    
    return result;
}

- (BOOL)refresh
{
//    [progressBar animate: nil];
//    [progressView displayIfNeeded];
    return [self runModalLoop];
}

- (void)setProgressTitleText:(NSString *)title
{
    [progressTitle setStringValue: title];
    [progressTitle setNeedsDisplay: YES];
}

- (void)setProgressItemText:(NSString *)item
{
    [progressItem setStringValue: item];
    [progressItem setNeedsDisplay: YES];
}

- (void)setProgressItemCount:(double)count
{
    [progressBar setMaxValue: (double)count];
    [progressBar setDoubleValue: 0.0];

    // Don't need to show the batch mode checkbox for single item
    if (count < 2.0 && [replaceConfirmApplyAll superview])
    {
        [replaceConfirmApplyAll retain];
        [replaceConfirmApplyAll removeFromSuperview];
    }
    else if ([replaceConfirmApplyAll superview] == nil)
    {
        [replaceConfirmView addSubview: replaceConfirmApplyAll];
        [replaceConfirmApplyAll release];
    }

    [progressBar setNeedsDisplay: YES];
}

- (BOOL)incrementProgress
{
    [progressBar incrementBy: 1.0];
    [progressBar setNeedsDisplay: YES];
    if ([progressBar doubleValue] == [progressBar maxValue])
        [self setProgressIndeterminate: YES];
    return [self runModalLoop];
}

- (void)setProgressIndeterminate:(BOOL)isIndeterminate
{
    [progressBar setIndeterminate: isIndeterminate];
    if (isIndeterminate)
    {
        [progressBar setUsesThreadedAnimation: YES];
        [progressBar setAnimationDelay: 0.5];
        [progressBar startAnimation: self];
    }
    else
    {
        [progressBar setUsesThreadedAnimation: NO];
        [progressBar stopAnimation: self];
    }

    [progressBar displayIfNeeded];
}


- (BOOL)showReplaceConfirmIfNecessaryForFile:(NSString *)aFile
{
    if (NSOnState != [replaceConfirmApplyAll state])
    {
        double interval = 0.1;
        BOOL isRunning;
        _replaceChoice  = kSquishNoChoice;

        [self showReplaceConfirm:YES forFile:aFile];

        while (1)
        {
            isRunning = [[NSRunLoop currentRunLoop] runMode:NSDefaultRunLoopMode beforeDate:[NSDate dateWithTimeIntervalSinceNow: interval]];
            if (kSquishNoChoice != _replaceChoice || !isRunning)
                break;
            if (_modalSession) [NSApp runModalSession: _modalSession];
        }

        [self showReplaceConfirm:NO forFile:nil];
        return YES;
    }

    return NO;
}

- (void)showReplaceConfirm:(BOOL)shouldShow forFile:(NSString *)aFile
{
    NSView *theView = [[self window] contentView];
    if (progressView == theView && shouldShow)
    {
        [[self window] setContentView: replaceConfirmView];
        [replaceConfirmTitle setStringValue:
            [NSString stringWithFormat:@"Another file with the name \"%@\" already exists. Do you wish to replace it?", aFile]];
    }
    else if (replaceConfirmView == theView && !shouldShow)
    {
        [[self window] setContentView: progressView];
    }
}

- (int)replaceChoice
{
    return _replaceChoice;
}


- (BOOL)showProfileConfirmIfNecessaryForFile:(NSString *)aFile
                         embeddedProfileName:(NSString *)aEmbedProfileName
{
    if (NSOnState != [profileConfirmApplyAll state])
    {
        double interval = 0.1;
        BOOL isRunning;
        _profileChoice  = kSquishNoChoice;
        [profileEmbeddedName setStringValue: aEmbedProfileName];

        [self showProfileConfirm:YES forFile:aFile];
        
        while (1)
        {
            isRunning = [[NSRunLoop currentRunLoop] runMode:NSDefaultRunLoopMode beforeDate:[NSDate dateWithTimeIntervalSinceNow: interval]];
            if (kSquishNoChoice != _profileChoice || !isRunning)
                break;
            if (_modalSession) [NSApp runModalSession: _modalSession];
        }
        
        [self showProfileConfirm:NO forFile:nil];
        return YES;
    }
    
    return NO;
}

- (void)showProfileConfirm:(BOOL)shouldShow forFile:(NSString *)aFile
{
    NSView *theView = [[self window] contentView];
    if (progressView == theView && shouldShow)
    {
        [[self window] setContentView: profileConfirmView];
        [profileConfirmTitle setStringValue:
            [NSString stringWithFormat:@"An embedded color profile was detected in \"%@\"", aFile]];
    }
    else if (profileConfirmView == theView && !shouldShow)
    {
        [[self window] setContentView: progressView];
    }
}

- (int)profileChoice
{
    return _profileChoice;
}

- (int)profileOverrideChoice
{
    return _profileOverrideChoice;
}


- (void)awakeFromNib
{
    _replaceChoice = kSquishNoChoice;
    _profileChoice = kSquishNoChoice;
    
    [profileOverrideMenu removeAllItems];
    NSMenu *profileMenu = [profileOverrideMenu menu];
    NSMenuItem *mi;
    mi = [profileMenu addItemWithTitle:@"No Profile" action:nil keyEquivalent:@""];
    [mi setTag: 0];
    [mi setToolTip: @"Process image as if it didn't have a color profile."];
    mi = [profileMenu addItemWithTitle:@"sRGB" action:nil keyEquivalent:@""];
    [mi setTag: cmSRGBData];
    [mi setToolTip: @"Substitute sRGB profile.\nUseful for images created on Windows."];    
}

- (void)windowDidLoad
{
    [[self window] setFrameAutosaveName: @"ProgressFrame"];
    [[self window] setContentView: progressView];
}

- (void)dealloc
{
    if ([replaceConfirmApplyAll superview] == nil)
        [replaceConfirmApplyAll release];
    if ([profileConfirmApplyAll superview] == nil)
        [profileConfirmApplyAll release];

    [super dealloc];
}

@end
